/*
 * ByteArray.cpp
 *
 *  Created on: Apr 12, 2014
 *      Author: cihan
 */

#include "ByteArray.h"

//#define DEBUG
#define TRACE_ERROR(x) printf("ERROR: %s:%d: ",__FUNCTION__, __LINE__); printf x; printf("\n");

ByteArray::ByteArray(): m_size(0),
						m_data(0),
						m_name(0)
{
//last change
//	m_size = 0;
//	m_data = NULL;
//	m_name = NULL;
}

ByteArray::~ByteArray()
{
	m_size = 0;

	if(!m_data)
	{
		delete[] m_data;
	}

	if(!m_name)
	{
		delete[] m_name;
	}
}

ByteArray::ByteArray(unsigned int size)
{
	m_size = size;
	m_data = new unsigned char[m_size];
	m_name = NULL;
}

unsigned int
ByteArray::Size()
{
	return m_size;
}

unsigned char*
ByteArray::Data()
{
	return m_data;
}

char *
ByteArray::Name()
{
	return m_name;
}

bool
ByteArray::SetDataFromHexString(const char * hex_string)
{
	if(!(hex_string[0] == '0' && (hex_string[1] == 'x' || hex_string[1] == 'X')))
	{
		TRACE_ERROR(("Hex string isn't valid"));
		return false;
	}

	if(m_data)
	{
		delete[] m_data;
	}

	m_size = (strlen(hex_string) -2 ) / 2 ;
	m_data = new unsigned char[m_size];
	if(!m_data)
	{
		TRACE_ERROR(("Out of memory"));
		m_size = 0;
		return false;
	}

	for(unsigned int i = 0; i < strlen(hex_string)-2; i++)
	{
		switch (hex_string[i+2])
		{
			case '0':
				if(i%2){m_data[i/2] |= 0x00;}else{m_data[i/2] = 0x00;}
		    	break;
			case '1':
				if(i%2){m_data[i/2] |= 0x01;}else{m_data[i/2] = 0x10;}
		    	break;
			case '2':
				if(i%2){m_data[i/2] |= 0x02;}else{m_data[i/2] = 0x20;}
		    	break;
			case '3':
				if(i%2){m_data[i/2] |= 0x03;}else{m_data[i/2] = 0x30;}
		    	break;
			case '4':
				if(i%2){m_data[i/2] |= 0x04;}else{m_data[i/2] = 0x40;}
		    	break;
			case '5':
				if(i%2){m_data[i/2] |= 0x05;}else{m_data[i/2] = 0x50;}
		    	break;
			case '6':
				if(i%2){m_data[i/2] |= 0x06;}else{m_data[i/2] = 0x60;}
		    	break;
			case '7':
				if(i%2){m_data[i/2] |= 0x07;}else{m_data[i/2] = 0x70;}
		    	break;
			case '8':
				if(i%2){m_data[i/2] |= 0x08;}else{m_data[i/2] = 0x80;}
		    	break;
			case '9':
				if(i%2){m_data[i/2] |= 0x09;}else{m_data[i/2] = 0x90;}
		    	break;
			case 'a':
			case 'A':
				if(i%2){m_data[i/2] |= 0x0a;}else{m_data[i/2] = 0xa0;}
		    	break;
			case 'b':
			case 'B':
				if(i%2){m_data[i/2] |= 0x0b;}else{m_data[i/2] = 0xb0;}
		    	break;
			case 'c':
			case 'C':
				if(i%2){m_data[i/2] |= 0x0c;}else{m_data[i/2] = 0xc0;}
		    	break;
			case 'd':
			case 'D':
				if(i%2){m_data[i/2] |= 0x0d;}else{m_data[i/2] = 0xd0;}
		    	break;
			case 'e':
			case 'E':
				if(i%2){m_data[i/2] |= 0x0e;}else{m_data[i/2] = 0xe0;}
		    	break;
			case 'f':
			case 'F':
				if(i%2){m_data[i/2] |= 0x0f;}else{m_data[i/2] = 0xf0;}
		    	break;
			default:
					return false;
		}

#ifdef DEBUG
		for(unsigned int i = 0; i < m_size; i++)
		{
			printf("%02X ", m_data[i]);
		}

		printf("\n");
#endif

	}

	return true;
}

bool
ByteArray::SetName(const char * name)
{
	if(!name)
	{
		return false;
	}

	if(m_name)
	{
		delete[] m_name;
	}

	m_name = new char[strlen(name)+1];
	if(!m_name)
	{
		TRACE_ERROR(("Out of memory"));
		return false;
	}

	strcpy(m_name, name);
	//memcpy(m_name, name, strlen(name)+1);

	return true;
}

//Deprecated
char *
ByteArray::GetName()
{
	return m_name;
}



void
ByteArray::Clear()
{
	if(m_data)
	{
		delete[] m_data;
		m_data = 0;
	}

	m_size = 0;
}



bool
ByteArray::FromData(unsigned char *data, unsigned int size)
{

	if(m_data)
	{
		delete[] m_data;
	}

	m_size = size;
	m_data = new unsigned char[m_size];
	if(!m_data)
	{
		TRACE_ERROR(("Out of memory"));
		m_size = 0;
		return false;
	}

	memcpy(m_data, data, m_size);

	return true;
}

bool
ByteArray::Clone(ByteArray byte_array)
{
	if(m_data)
	{
		delete[] m_data;
	}

	m_size = byte_array.Size();
	m_data = new unsigned char[m_size];
	if(!m_data)
	{
		TRACE_ERROR(("Out of memory"));
		m_size = 0;
		return false;
	}

	this->SetName(byte_array.GetName());
	memcpy(m_data, byte_array.Data(), m_size);

	return true;
}

bool
ByteArray::PrintAsHexString()
{
	if(m_size <= 0)
	{
		TRACE_ERROR(("size of data <= 0"));
		return false;
	}

	for(unsigned int i = 0; i < m_size; i++)
	{
		printf("%02X ", m_data[i]);
	}

	return true;
}

bool
ByteArray::AddByteArrayToEnd(ByteArray byte_array)
{
	unsigned int temp_size = m_size + byte_array.Size();
	unsigned char * temp_data = new unsigned char[temp_size];
	if(!temp_data)
	{
		TRACE_ERROR(("Out of memory"));
		return false;
	}

	memcpy(temp_data, m_data, m_size);
	memcpy(temp_data+m_size, byte_array.Data(), byte_array.Size());

	if(m_data)
	{
		delete[] m_data;
	}

	m_data = temp_data;
	m_size = temp_size;

	return true;
}

bool
ByteArray::RemoveByteArrayFromBegin(ByteArray &byte_array, unsigned int size)
{
	byte_array.FromData(m_data, size);

	m_size -=size;

//	unsigned char * temp_data = new unsigned char[m_size];
//	if(!temp_data)
//	{
//		TRACE_ERROR(("Out of memory"));
//		m_size +=size;
//		return false;
//	}
//
//	memcpy(temp_data,m_data+size, m_size);
//	memcpy(m_data,temp_data, m_size);

	unsigned char * temp_data = new unsigned char[m_size];
	if(!temp_data)
	{
		TRACE_ERROR(("Out of memory"));
		m_size +=size;
		return false;
	}

	memcpy(temp_data,m_data+size, m_size);

	if(m_data)
	{
		delete[] m_data;
	}

	m_data = temp_data;

	return true;
}

bool
ByteArray::LogicOrWith(ByteArray byte_array)
{
	//m_data     01101000 01010011 10101110
	//byte_array          00000010 01011100
	//result 			  01010011 11111110
	if(m_size > byte_array.Size())
	{
		for(unsigned int i = 1; i <= byte_array.Size(); i++)
		{
			m_data[m_size - i] |= byte_array.Data()[byte_array.Size()-i];
		}
	}
	else
	{
		for(unsigned int i = 1; i <= m_size; i++)
		{
			byte_array.Data()[byte_array.Size()-i] |= m_data[m_size - i];
		}
	}

	return true;
}

bool
ByteArray::LogicAndWith(ByteArray byte_array)
{

	if(m_size > byte_array.Size())
	{
		for(unsigned int i= 1; i <= byte_array.Size(); i++)
		{
			m_data[m_size - i] &= byte_array.Data()[byte_array.Size()-i];
		}
	}
	else
	{
		for(unsigned int i = 1; i <= m_size; i++)
		{
			byte_array.Data()[byte_array.Size()-i] &= m_data[m_size - i];
		}
	}

	return true;
}

bool
ByteArray::LeftCircularShift(unsigned int number_of_bits)
{
	unsigned int shifted_byte_size = number_of_bits / 8;
	unsigned int shifted_bit = number_of_bits % 8;
	unsigned char shifted_temp = 0;
	unsigned char temp = 0;

	shifted_temp = m_data[m_size-1];
	for(unsigned int i = 1; i < m_size; i++)
	{
		shifted_temp >>= (8-shifted_bit);

		temp = m_data[m_size-i-1];
		m_data[m_size-i-1] <<= 2;
		m_data[m_size-i-1] |= shifted_temp;

		shifted_temp = temp;
	}

	shifted_temp >>= (8-shifted_bit);

	temp = m_data[m_size-1];
	m_data[m_size-1] <<= 2;
	m_data[m_size-1] |= shifted_temp;

	if(shifted_byte_size % m_size != 0 )
	{
		unsigned char *temp_buffer = new unsigned char[m_size];
		if(!temp_buffer)
		{
			TRACE_ERROR(("Out of memory"));
			return false;
		}

		memcpy(temp_buffer, m_data+shifted_byte_size, (m_size - shifted_byte_size));
		memcpy(temp_buffer+(m_size - shifted_byte_size), m_data, shifted_byte_size);

		if(m_data)
		{
			delete m_data;
		}

		m_data = temp_buffer;
	}

	return true;
}

bool
ByteArray::RightCircularShift(unsigned int number_of_bits)
{
	unsigned int shifted_byte_size = number_of_bits / 8;
	unsigned int shifted_bit = number_of_bits % 8;
	unsigned char shifted_temp = 0;
	unsigned char temp = 0;

	shifted_temp = m_data[0];

	for(unsigned int i = 1; i < m_size; i++)
	{
		shifted_temp <<= (8-shifted_bit);

		temp = m_data[i];
		m_data[i] >>= 2;
		m_data[i] |= shifted_temp;

		shifted_temp = temp;
	}

	shifted_temp <<= (8-shifted_bit);

	temp = m_data[0];
	m_data[0] >>= 2;
	m_data[0] |= shifted_temp;

	if(shifted_byte_size % m_size != 0 )
	{
		unsigned char *temp_buffer = new unsigned char[m_size];
		if(!temp_buffer)
		{
			TRACE_ERROR(("Out of memory"));
			return false;
		}

		memcpy(temp_buffer, m_data+(m_size-shifted_byte_size), shifted_byte_size);
		memcpy(temp_buffer+shifted_byte_size, m_data, (m_size-shifted_byte_size));

		if(m_data)
		{
			delete m_data;
		}

		m_data = temp_buffer;
	}

	return true;
}

bool
ByteArray::IsEqual(ByteArray byte_array)
{
	if(m_size != byte_array.Size())
	{
		return false;
	}

	if(memcmp(m_data, byte_array.Data(), byte_array.Size()) != 0)
	{
		return false;
	}

	return true;
}

unsigned int
ByteArray::ToUnsignedInt()
{
	unsigned int temp = 0;

	if(sizeof(unsigned int) <= m_size)
	{
		for(unsigned int i = 0; i < sizeof(unsigned int); i++)
		{
			//TODO:: check and fix this
			temp <<= 8;
			temp |= m_data[m_size-sizeof(unsigned int)+i];
		}
	}
	else
	{
		for(unsigned int i = 0; i < m_size; i++)
		{
			temp <<= 8;
			temp |= m_data[i];
		}
	}

	return temp;
}

bool
ByteArray::Count(bool resizeable)
{
	unsigned char value = 0;

	for( int i = m_size-1; i >= 0; i--)
	{
		m_data[i] += 1;

		if(memcmp(&m_data[i], &value, 1) != 0)
		{
			break;
		}

		if((i == 0) && resizeable)
		{
			unsigned char * resizeable = new unsigned char[m_size+1];
			if(!resizeable)
			{
				TRACE_ERROR(("Out of memory"));
				return false;
			}

			memcpy(resizeable+1, m_data, m_size);

			resizeable[0] = 1;

			if(m_data)
			{
				delete[] m_data;
			}


			m_data = resizeable;

			m_size++;
		}
	}

	return true;
}
