/*
 * Functions.cpp
 *
 *  Created on: Apr 13, 2014
 *      Author: cihan
 */

#include "Functions.h"
#include "ByteArray.h"

#include <openssl/aes.h>
#include <openssl/evp.h>
#include <openssl/ec.h>
#include <openssl/rand.h>

#define TRACE_ERROR(x) printf("ERROR: %s:%d: ",__FUNCTION__, __LINE__); printf x; printf("\n");


static ByteArray buffer;

Functions::Functions() {
	// TODO Auto-generated constructor stub

}

Functions::~Functions() {
	// TODO Auto-generated destructor stub
}

bool
Functions::GenerateRandom(ByteArray size_value, ByteArray &answer)
{
	unsigned char * data = new unsigned char[size_value.ToUnsignedInt()];
	if(!data)
	{
		TRACE_ERROR(("Out of memory"));
		return false;
	}

	RAND_bytes(data, (int)size_value.ToUnsignedInt());

	answer.FromData(data, size_value.ToUnsignedInt());

	if(data)
	{
		delete[] data;
	}

	return true;
}

bool
Functions::Concatenate(ByteArray input, ByteArray input_2, ByteArray & answer)
{
	answer.Clone(input);
	answer.AddByteArrayToEnd(input_2);
	return true;
}

bool
Functions::Pseudorandom(ByteArray data, ByteArray key, ByteArray & answer)
{
//	printf("data:\n");
//	data.PrintAsHexString();
//	printf("\n");
//
//	printf("key:\n");
//	key.PrintAsHexString();
//	printf("\n");

	char *digest = new char[key.Size()];
	memset(digest, 0 , key.Size());
	if(!digest)
	{
		TRACE_ERROR(("Out of memory"));
		return false;
	}

	aes_xcbc_mac_128(key.Data(), key.Size(), data.Data(), data.Size(), digest);

//	printf("\n");
//	for(unsigned int i = 0; i < key.Size(); i++)
//		printf("%02X ", (unsigned char) digest[i]);
//	printf("\n");

	answer.FromData((unsigned char*)digest, key.Size());

	if(digest)
	{
		delete[] digest;
	}
//	printf("answer:\n");
//	answer.PrintAsHexString();
//	printf("\n");

	return true;
}

bool
Functions::PseudorandomPlus(ByteArray data, ByteArray key, ByteArray & answer)
{
	answer.Clear();

	unsigned int iteration_size = 5;

	unsigned char *result = new unsigned char[key.Size() * iteration_size];
	if(!result)
	{
		TRACE_ERROR(("Out of memory"));
		return false;
	}

	char *digest = new char[key.Size()];
	if(!digest)
	{
		TRACE_ERROR(("Out of memory"));
		return false;
	}

	ByteArray temp_answer;
	ByteArray new_sequence;
	ByteArray iteration;

	for(iteration.SetDataFromHexString("0x00"); iteration.ToUnsignedInt() < iteration_size; iteration.Count(false))
	{
		new_sequence.AddByteArrayToEnd(temp_answer);
		new_sequence.AddByteArrayToEnd(data);
		new_sequence.AddByteArrayToEnd(iteration);

		Pseudorandom(new_sequence, key, temp_answer);

		answer.AddByteArrayToEnd(temp_answer);
	}

	return true;
}

bool
Functions::Ecdh(ByteArray &answer)
{
	EC_KEY *key; //, *peerkey;
	//EC_GROUP *ec_group;
	//int field_size;
	//unsigned char *secret;
	//size_t secret_len = 0;

	/* Create an Elliptic Curve Key object and set it up to use the ANSI X9.62 Prime 256v1 curve */
	key = EC_KEY_new_by_curve_name(NID_X9_62_prime256v1);
	//peerkey = EC_KEY_new_by_curve_name(NID_X9_62_prime256v1);

	 /* Generate the private and public key*/
	 EC_KEY_generate_key(key);

	 unsigned char public_key[kUncompressedP256PointBytes];
	 if(EC_POINT_point2oct(EC_KEY_get0_group(key), EC_KEY_get0_public_key(key), POINT_CONVERSION_UNCOMPRESSED, public_key,
	                    sizeof(public_key), NULL) != sizeof(public_key))
	 {
		 TRACE_ERROR(("Convention error"));
		 return false;
	 }

	 return answer.FromData(public_key, kUncompressedP256PointBytes);
}

bool
Functions::Grab(ByteArray &data, ByteArray size, ByteArray &answer)
{
	return data.RemoveByteArrayFromBegin(answer, size.ToUnsignedInt());
}

bool
Functions::Encrypte(ByteArray data, ByteArray key, ByteArray &answer)
{
    unsigned char * enc_out = new unsigned char[data.Size()];
	if(!enc_out)
	{
		TRACE_ERROR(("Out of memory"));
		return false;
	}

    AES_KEY enc_key;

    AES_set_encrypt_key(key.Data(), 128, &enc_key);
    AES_encrypt(data.Data(), enc_out, &enc_key);

    answer.FromData(enc_out, data.Size());

    if(enc_out)
    {
    	delete[] enc_out;
    }

    return true;
}

bool
Functions::Decrypte(ByteArray data, ByteArray key, ByteArray &answer)
{
    unsigned char * dec_out = new unsigned char[data.Size()];
	if(!dec_out)
	{
		TRACE_ERROR(("Out of memory"));
		return false;
	}

    AES_KEY dec_key;

    AES_set_decrypt_key(key.Data(),128,&dec_key);
    AES_decrypt(data.Data(), dec_out, &dec_key);

    answer.FromData(dec_out, data.Size());

    if(dec_out)
    {
    	delete[] dec_out;
    }

    return true;
}

bool
Functions::Compare(ByteArray input, ByteArray input_2, ByteArray &answer)
{
	//return input.IsEqual(input_2);

    answer.Clear();
    if(input.IsEqual(input_2))
    {
        answer.SetDataFromHexString("0x01");
    }
    else
    {
        answer.SetDataFromHexString("0x00");
    }
    return true;
}

bool
Functions::SendData(ByteArray input)
{
	buffer.Clear();
	buffer.Clone(input);

	return true;
}

bool
Functions::ReceiveData(ByteArray &answer)
{
	answer.Clone(buffer);

	return true;
}

bool
Functions::IfFunc(bool &bool_if , ByteArray mpz_value, ByteArray &answer)
{
    if(mpz_value.ToUnsignedInt())
    {
        bool_if = false;
    }
    else
    {
        bool_if = true;
    }
    return true;
}

void
Functions::aes_xcbc_mac_128(unsigned char *key, int key_len, unsigned char *text, int text_len, char *digest)
{
	AES_KEY aes_key, aes_key_K1;
	unsigned char K1[16];
	unsigned char K2[16];
	unsigned char K3[16];
	unsigned char Ebuf1[16], Ebuf2[16];
	unsigned char *e_prev, *e_curr;
	int blocks;
	int left;
	int i, pos;
	unsigned char *data;

	//LOG_FUNC_START(1);

	/* By definition key_len MUST be 128 bits i.e. 16 bytes.
	 * Text len must not be negative.
	 */
	//LOG_TRACE("aes_xcbc_mac_128: key_len = %d, text_len = %d.\n",
	//		key_len, text_len);
	//g_assert(key_len == 16 && text_len >= 0);

	blocks = text_len / 16;
	left   = text_len % 16;
	if(!left) blocks--;

	memset(K1, 0x01, 16);
	memset(K2, 0x02, 16);
	memset(K3, 0x03, 16);

	AES_set_encrypt_key(key, key_len*8, &aes_key);
	AES_encrypt(&K1[0],&K1[0],&aes_key);
	AES_encrypt(&K2[0],&K2[0],&aes_key);
	AES_encrypt(&K3[0],&K3[0],&aes_key);

	AES_set_encrypt_key(&K1[0], 16*8, &aes_key_K1);
	e_prev = &Ebuf1[0];
	e_curr = &Ebuf2[0];
	memset(e_prev, 0x00, 16);

	if(!text_len) {
		e_curr[0] = (unsigned char)0x80 ^ K3[0];
		for(pos = 1; pos < 16; pos++) {
			e_curr[pos] = K3[pos];
		}
		AES_encrypt(e_curr,e_prev,&aes_key_K1);
	} else {
		data = text;
		for(i = 0; i < blocks; i++) {
			for(pos = 0; pos < 16; pos++) {
				e_curr[pos] = e_prev[pos] ^ data[pos];
			}
			data += 16;
			AES_encrypt(e_curr,e_prev,&aes_key_K1);
		}

		if( !left ) {
			for(pos = 0; pos < 16; pos++) {
				e_curr[pos] = e_prev[pos] ^ data[pos] ^ K2[pos];
			}
			AES_encrypt(e_curr,e_prev,&aes_key_K1);
		} else {
			for(pos = 0; pos < left; pos++) {
				e_curr[pos] = data[pos];
			}
			e_curr[left] = (unsigned char)0x80;
			for(pos = left+1; pos < 16; pos++) {
				e_curr[pos] = (unsigned char)0x00;
			}
			for(pos = 0; pos < 16; pos++) {
				e_curr[pos] = e_prev[pos] ^ e_curr[pos] ^ K3[pos];
			}
			AES_encrypt(e_curr,e_prev,&aes_key_K1);
		}
	}
	memcpy(digest, e_prev, 16);
}
