/*
 * Functions.h
 *
 *  Created on: Apr 13, 2014
 *      Author: cihan
 */

#ifndef FUNCTIONS_H_
#define FUNCTIONS_H_

class ByteArray;

class Functions {
	enum {
		// A P-256 field element consists of 32 bytes.
		kP256FieldBytes = 32,
		// A P-256 point in uncompressed form consists of 0x04 (to denote
		// that the point is uncompressed) followed by two, 32-byte field
		// elements.
		kUncompressedP256PointBytes = 1 + 2 * kP256FieldBytes,
		// The first byte in an uncompressed P-256 point.
		kUncompressedECPointForm = 0x04,
	};
public:
	Functions();
	virtual ~Functions();

	bool GenerateRandom(ByteArray size_value, ByteArray &answer);
	bool Concatenate(ByteArray input, ByteArray input_2, ByteArray & answer);
	bool Pseudorandom(ByteArray data, ByteArray key, ByteArray & answer);
	bool PseudorandomPlus(ByteArray data, ByteArray key, ByteArray &answer);
	bool Ecdh(ByteArray &answer);
	bool Grab(ByteArray &data, ByteArray size, ByteArray &answer);
	bool Encrypte(ByteArray data, ByteArray key, ByteArray &answer);
	bool Decrypte(ByteArray data, ByteArray key, ByteArray &answer);
	bool Compare(ByteArray input, ByteArray input_2, ByteArray &answer);

	bool SendData(ByteArray input);
	bool ReceiveData(ByteArray &answer);
	bool IfFunc(bool &bool_if , ByteArray mpz_value, ByteArray &answer);
private:
	void aes_xcbc_mac_128(unsigned char *key, int key_len, unsigned char *text, int text_len, char *digest);
};

#endif /* FUNCTIONS_H_ */
