#include "expression/variablelist.h"
#include "expression/Functions.h"
#include "expression/parser.h"
#include "Device.h"
#include "FileParser.h"

//#define UNITTEST

#include "stdio.h"
#include <iostream>
//#include <gtest/gtest.h>
int main(int argc, char* argv[])
{
#ifdef UNITTEST

	::testing::InitGoogleTest(&argc, argv);
	return RUN_ALL_TESTS();
#else
#if 0
	printf("TEST BEGIN\n");
	printf("****BYTEARRAY TEST BEGIN****\n\n");
	ByteArray count;

	printf("Count: 0xFFFF and resizeable EXPECTED: \n01 00 00\n");
	count.FromHexString("0xFFFF");
	count.Count(true); count.PrintAsHexString(); printf("\n\n");
	printf("Count: 0xFFFF and not resizeable EXPECTED: \n00 00\n");
	count.FromHexString("0xFFFF");
	count.Count(false); count.PrintAsHexString(); printf("\n\n");
	printf("****BYTEARRAY TEST END****\n\n");

	printf("Variable List TEST BEGIN\n");
	Variablelist vl_test;
	ByteArray ba01;
	ByteArray ba02;
	ByteArray ba03;
	ByteArray ba04;

	ba01.FromHexString("0x0102");
	vl_test.add("var1", ba01);
	vl_test.get_value("var1", ba02); ba02.PrintAsHexString(); printf("\n");
	ba03.FromHexString("0x0103");
	vl_test.add("var1", ba03);
	vl_test.get_value("var1", ba04); ba04.PrintAsHexString(); printf("\n");

	printf("Variable List TEST END\n\n");

	printf("****FUNCTION TEST BEGIN****\n\n");
	Functions functest;

	ByteArray generaterandom_size;
	ByteArray generaterandom_answer;
	ByteArray concatenate_input1;
	ByteArray concatenate_input2;
	ByteArray concatenate_answer;
	ByteArray pseudorandom_data;
	ByteArray pseudorandom_key;
	ByteArray pseudorandom_answer;
	ByteArray pseudorandomplus_answer;
	ByteArray ecdh_answer;
	ByteArray aes_data_plain;
	ByteArray aes_data_cipher;
	ByteArray aes_key;
	ByteArray aes_answer;
	ByteArray send_data;
	ByteArray receive_data;
	ByteArray grab_data;
	ByteArray grab_size;
	ByteArray grab_answer;

	printf("Function GenerateRandom - Size: 0x01 EXPECTED: \n--\n");
	generaterandom_size.FromHexString("0x01");
	functest.GenerateRandom(generaterandom_size, generaterandom_answer); generaterandom_answer.PrintAsHexString(); printf("\n\n");

	printf("Function Concatenate - Input: 0x01A5 - Input: 0x03A506 - EXPECTED: \n01 A5 03 A5 06\n");
	concatenate_input1.FromHexString("0x01A5");
	concatenate_input2.FromHexString("0x03A506");
	functest.Concatenate(concatenate_input1, concatenate_input2, concatenate_answer); concatenate_answer.PrintAsHexString(); printf("\n\n");

	printf("Function Pseudorandom - \nData: 0x000102030405060708090a0b0c0d0e0f10111213\nKey: 0x000102030405060708090a0b0c0d0e0f\n"
			"EXPECTED: \n47 f5 1b 45 64 96 62 15 b8 98 5c 63 05 5e d3 08\n");
	pseudorandom_data.FromHexString("0x000102030405060708090a0b0c0d0e0f10111213");
	pseudorandom_key.FromHexString("0x000102030405060708090a0b0c0d0e0f");
	functest.Pseudorandom(pseudorandom_data, pseudorandom_key, pseudorandom_answer); pseudorandom_answer.PrintAsHexString(); printf("\n\n");

	printf("Function PseudorandomPlus - \nData: 0x000102030405060708090a0b0c0d0e0f10111213\nKey: 0x000102030405060708090a0b0c0d0e0f\n"
				"EXPECTED: \n??\n");
	functest.PseudorandomPlus(pseudorandom_data, pseudorandom_key, pseudorandomplus_answer); pseudorandomplus_answer.PrintAsHexString(); printf("\n\n");

	printf("Function Ecdh - \nEXPECTED: \n??\n");
	functest.Ecdh(ecdh_answer); ecdh_answer.PrintAsHexString(); printf("\n\n");

	printf("Function Aes - Encrypte \nData: 0x6bc1bee22e409f96e93d7e117393172a\nKey: 0x2b7e151628aed2a6abf7158809cf4f3c\n"
			"EXPECTED: \n3a d7 7b b4 0d 7a 36 60 a8 9e ca f3 24 66 ef 97\n");
	aes_data_plain.FromHexString("0x6bc1bee22e409f96e93d7e117393172a");
	aes_key.FromHexString("0x2b7e151628aed2a6abf7158809cf4f3c");
	functest.Encrypte(aes_data_plain, aes_key, aes_answer); aes_answer.PrintAsHexString(); printf("\n\n");

	printf("Function Aes - Decrypte \nData: 0x3ad77bb40d7a3660a89ecaf32466ef97\nKey: 0x2b7e151628aed2a6abf7158809cf4f3c\n"
			"EXPECTED: \n6b c1 be e2 2e 40 9f 96 e9 3d 7e 11 73 93 17 2a\n");
	aes_data_cipher.FromHexString("0x3ad77bb40d7a3660a89ecaf32466ef97");
	aes_key.FromHexString("0x2b7e151628aed2a6abf7158809cf4f3c");
	functest.Decrypte(aes_data_cipher, aes_key, aes_answer); aes_answer.PrintAsHexString(); printf("\n\n");

	printf("Function Compare - aes_plain and aes_out_of_dec \nEXPECTED: \nTrue\n");
	printf("%s\n\n", (functest.Compare(aes_data_plain, aes_answer) ? "True" : "False"));

	printf("Function SendData. Then Receive Data and compare both of them \nEXPECTED: \nTrue\n");
	send_data.FromHexString("0x04050307AB");
	functest.SendData(send_data);
	functest.ReceiveData(receive_data);
	printf("%s\n\n", (functest.Compare(send_data, receive_data) ? "True" : "False"));

	printf("Function Grab - Grab Data: 0x03A506 - Size: 0x02 - EXPECTED: \n03 A5 - 06\n");
	grab_data.FromHexString("0x03A506");
	grab_size.FromHexString("0x02");
	functest.Grab(grab_data, grab_size, grab_answer); grab_answer.PrintAsHexString(); printf("- "); grab_data.PrintAsHexString(); printf("\n\n");

	printf("****FUNCTION TEST END****\n\n");

	printf("****FUNCTIONAL TEST BEGIN****\n\n");
	Parser prs;

	ByteArray parse_answer;

	printf("Parser test=0x01020304 \nEXPECTED: \n01 02 03 04\n");
	prs.parse("test=0x01020304", parse_answer); parse_answer.PrintAsHexString(); printf("\n\n");

	printf("Parser kes=generaterandom(0x02) \nEXPECTED: \n-- --\n");
	prs.parse("kes=generaterandom(0x02)", parse_answer); parse_answer.PrintAsHexString(); printf("\n\n");

	printf("Parser ns=generaterandom(0x02) \nEXPECTED: \n-- --\n");
	prs.parse("ns=generaterandom(0x02)", parse_answer); parse_answer.PrintAsHexString(); printf("\n\n");

	printf("Parser tempkey=prf(0x000102030405060708090a0b0c0d0e0f10111213 , 0x000102030405060708090a0b0c0d0e0f) \nEXPECTED: \n-- --\n");
	prs.parse("tempkey=prf(0x000102030405060708090a0b0c0d0e0f10111213 , 0x000102030405060708090a0b0c0d0e0f)", parse_answer); parse_answer.PrintAsHexString(); printf("\n\n");

	printf("Parser ecdh_value=ecdh(0x00) \nEXPECTED: \n-- --\n");
	prs.parse("ecdh_value=ecdh(0x00)", parse_answer); parse_answer.PrintAsHexString(); printf("\n\n");

	printf("Parser grabdata=0x03A506 \nEXPECTED: \n03 A5 06\n");
	prs.parse("grabdata=0x03A506", parse_answer); parse_answer.PrintAsHexString(); printf("\n\n");

	printf("Parser grab_ans = grab(grabdata, 0x02) \nEXPECTED: \n03 A5\n");
	prs.parse("grab_ans = grab(grabdata, 0x02)", parse_answer); parse_answer.PrintAsHexString(); printf("\n\n");

	printf("Parser remain_data = grabdata \nEXPECTED: \n06\n");
	prs.parse("remain_data  = grabdata", parse_answer); parse_answer.PrintAsHexString(); printf("\n\n");

	printf("****FUNCTIONAL TEST END****\n\n");

	printf("TEST END\n");
#else

    queue<unsigned int> execute_order_of_devices;
    deque<Device*> devices;

    FileParser fileParser;
    fileParser.ParseFile(execute_order_of_devices, devices);

    printf("INFO: number of jobs [%d], number of devices [%d]\n", execute_order_of_devices.size(), devices.size());

    for(unsigned int i = 0; i < devices.size(); i++)
    {
    	printf("Device[%d].name: %s\n", i, devices[i]->GetDeviceName());
    }

    unsigned int size_of_order = execute_order_of_devices.size();

    for(unsigned int i = 0; i < size_of_order; i++)
    {
    	devices[execute_order_of_devices.front()]->ExecuteNextProcess();
    	execute_order_of_devices.pop();
    }

#endif
    return 0;
#endif

}


